@@ -1,6 +1,8 @@
 .composition <- function(LHS, RHS, attributes) {
 
   # logic_name <- tolower(fuzzy_logic()$name)
+  LHS <- methods::as(LHS, "dgCMatrix")
+  RHS <- methods::as(RHS, "dgCMatrix")
 
   equal_LHS <- .equal_sets(LHS)
 
