@@ -21,7 +21,7 @@
 
 }
 
-.concept_to_string <- function(C, objects, attributes) {
+.old.concept_to_string <- function(C, objects, attributes) {
 
   A <- .set_to_string(C$get_extent()$get_vector(), objects)
   B <- .set_to_string(C$get_intent()$get_vector(), attributes)
@@ -29,3 +29,12 @@
   return(paste0("(", A, ", ", B, ")"))
 
 }
+
+.concept_to_string <- function(vA, vB, objects, attributes) {
+
+  A <- .set_to_string(vA, objects)
+  B <- .set_to_string(vB, attributes)
+
+  return(paste0("(", A, ", ", B, ")"))
+
+}
